#!/usr/bin/env Rscript


# Goal: Parse kraken results: Weight by contig length


# define

kraken2.file <- snakemake@input[["taxonkit_out"]]

outfile <- snakemake@output[["summary"]]

taxlevel <- snakemake@params[["taxlevel"]]
  

# load data

kraken2 <- read.delim(kraken2.file, stringsAsFactors = F, header = F)
colnames(kraken2) <- c("contig","taxid","length","lineage")



if (taxlevel == "genus"){
  tax_delimter <- "|g__"
} else if (taxlevel == "species"){
  tax_delimter <- "|s__"
} else {
  warning(paste("Undefined parameter value: taxlevel = ",taxlevel,"Using species as taxonomic level for kraken2 contig classification"))
  tax_delimter <- "|s__"
}

kraken2$species <- sapply(kraken2$lineage,function(x) strsplit(x,tax_delimter,fixed = T)[[1]][2])



# analysis ------

species_set <- unique(kraken2$species)
# remove NA
species_set <- species_set[ !is.na(species_set)]


species_cumlength <- sapply(species_set, function(myspecies){
  species_sum <- sum(kraken2$length[kraken2$species == myspecies], na.rm = T)
  return(species_sum)
})


species_fraction <- species_cumlength/sum(species_cumlength)


kraken_contigstats <- data.frame(species=names(species_cumlength),species_cumlength=species_cumlength,species_fraction=species_fraction)
kraken_contigstats <- kraken_contigstats[order(kraken_contigstats$species_fraction, decreasing = T),]


# write results ---------------------------
write.table(kraken_contigstats, outfile, sep="\t", quote = F, row.names = F)






